#
# note_cgi.py - Note CGI methods.
# Copyright (c) 2005 shinGETsu Project.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: note_cgi.py,v 1.16 2005/10/28 13:01:41 fuktommy Exp $
#

import cgi
import re

import config
import gateway
from cache import *

__version__ = "$Revision: 1.16 $"


class CGI(gateway.CGI):

    """Class for /note.cgi."""

    appli_type = "note"

    def run(self):
        path = self.path_info()
        form = cgi.FieldStorage(environ=self.environ, fp=self.stdin)

        if form.getfirst("cmd", "") == "post" and \
           form.getfirst("file", "").startswith("note_") and \
           self.environ["REQUEST_METHOD"] == "POST":
            self.do_post(path, form)
            datfile = form.getfirst("file", "")
            title = self.str_encode(self.file_decode(datfile))
            self.print302(self.note_cgi + self.sep + title)
        elif (path == "") or (path.find("/") > 0):
            self.print404()
        elif form.getfirst("mode", "") == "edit":
            self.print_edit(path)
        elif form.getfirst("mode", "") == "history":
            self.print_history(path)
        else:
            self.print_note(path)

    def print_note(self, path):
        cache = Cache(self.file_encode("note", path))
        if cache.has_record():
            pass
        elif self.check_get_cache():
            self.get_cache(cache)
        else:
            self.print404()
            return
        self.header(path)
        (buf, conflict) = self.read_note(cache)
        if (buf is not None) and ("body" in buf):
            body = self.html_format(buf["body"])
        else:
            body = ""
        self.stdout.write("<p>\n    %s<br />\n</p>" % body)
        if conflict:
            self.stdout.write("<p><em>%s</em></p>" % self.message["conflict"])
        self.stdout.write(
            '<p>\n' +
            '  <a href="' + self.note_cgi + "/" + self.str_encode(path) +
            '?mode=edit">' + self.message["edit"] + '</a> |\n' +
            '  <a href="' + self.note_cgi + "/" + self.str_encode(path) +
            '?mode=history">' + self.message["history"] + '</a>\n' +
            '</p>')
        self.menubar()
        self.remove_file_form(cache)
        self.footer()

    def read_note(self, cache):
        """Read note history."""
        buf = None
        ref = {}
        for r in cache:
            rec = cache[r]
            rec.load()
            if "remove_stamp" in rec:
                continue
            buf = rec
            if "base_stamp" in rec:
                ref[rec.idstr] = 0
            if ("base_stamp" in rec) and ("base_id" in rec):
                base = rec["base_stamp"] + "_" + rec["base_id"]
                if base in ref:
                    ref[base] += 1
        orphan = 0
        for key in ref:
            if ref[key] == 0:
                orphan += 1
        return (buf, (orphan>1))

    def print_edit(self, path):
        """Print edit page."""
        cache = Cache(self.file_encode("note", path))
        if not re.search(config.robot, self.environ["HTTP_USER_AGENT"]):
            self.get_cache(cache)
        self.header(path, deny_robot=True)
        (buf, conflict) = self.read_note(cache)
        if (buf is not None) and ("body" in buf):
            body = self.html_format(buf["body"])
        else:
            body = ""
        self.stdout.write("<p>\n    %s<br />\n</p>" % body)
        self.print_edit_form(cache, buf)
        if conflict:
            self.stdout.write("<p><em>%s</em></p>" % self.message["conflict"])
        self.remove_file_form(cache)
        self.footer()

    def print_history(self, path):
        """Print note history."""
        cache = Cache(self.file_encode("note", path))
        if not re.search(config.robot, self.environ["HTTP_USER_AGENT"]):
            self.get_cache(cache)
        self.header(path, deny_robot=True)
        if self.isadmin:
            self.stdout.write(
                '<form method="get" action="' + self.admin_cgi + '">\n' +
                '<p>\n' +
                '  <input type="hidden" name="cmd" value="delete" />\n' +
                '  <input type="hidden" name="file"' +
                ' value="'+ cache.datfile + '" />\n' +
                '</p>\n')
        self.stdout.write('<dl>\n')

        recs = []
        ref = {}
        for r in cache:
            rec = cache[r]
            rec.load()
            recs.append(rec)
            ref[rec.idstr] = 0
            if ("base_stamp" in rec) and ("base_id" in rec):
                base_idstr = rec["base_stamp"] + "_" + rec["base_id"]
                if base_idstr not in ref:
                    ref[base_idstr] = 1
                else:
                    ref[base_idstr] += 1
            else:
                ref[rec.idstr] = 1
                continue

        if recs:
            ref[recs[-1].idstr] = 1

        for rec in recs:
            stamp = self.localtime(rec["stamp"])

            if "body" in rec:
                body = self.html_format(rec["body"])
            else:
                body = ""

            base = ""
            if "base_id" in rec:
                base = rec["base_id"][:8]
                base = cgi.escape(base, True)
                if base:
                    base = '(<a href="#r%s">&gt;&gt;%s</a>)' % (base, base)

            remove = ""
            if ("remove_id" in rec) and ("remove_stamp" in rec):
                id = rec["remove_id"][:8]
                remove = '[[%s: <a href="#r%s">%s</a>]]<br />' % \
                         (self.message["remove"], id, id)

            id = rec["id"][:8]
            if ref[rec.idstr] > 0:
                conflict = ""
            else:
                conflict = "    <em>%s</em>\n" % self.message["conflict"]

            self.stdout.write('  <dt id="r' + id + '">\n')
            if self.isadmin:
                self.stdout.write(
                    '    <input type="radio" name="record" value="' +
                    rec.idstr + '" tabindex="1" accesskey="s" />\n')
            self.stdout.write(
                '    ' + id + "\n" +
                '    ' + stamp + "\n" +
                '    ' + base + "\n" +
                conflict +
                '  </dt>\n' +
                '  <dd>' + body + '<br />' + remove + "<br /></dd>\n")

        self.stdout.write("</dl>\n")
        if self.isadmin:
            self.stdout.write(
                "<p>\n" +
                '  <input type="submit" value="' + self.message["del_record"] +
                '" tabindex="2" accesskey="d" />\n' +
                "</p>\n" +
                "</form>\n")

	(buf, conflict) = self.read_note(cache)
	self.print_edit_form(cache, buf)
        self.remove_file_form(cache)
        self.footer()

    def print_edit_form(self, cache, rec):
        if (rec is not None) and ("body" in rec):
            body = re.sub(r"<br>", "\n", rec["body"])
            stamp = rec["stamp"]
            id = rec["id"]
        else:
            body = ""
            stamp = ""
            id = ""

        self.stdout.write(
            '<form method="post" action="' + self.note_cgi + '"><p>\n' +
            '  <input type="hidden" name="cmd" value="post" />\n' +
            '  <input type="hidden" name="file"' +
            ' value="' + cache.datfile + '" />\n' +
            '  <input type="submit" value="' + self.message["post"] + '"' +
            ' name="submit" tabindex="1" accesskey="w" />\n' +
            '  <input type="checkbox" name="dopost" value="dopost"' +
            ' checked="checked" tabindex="2" accesskey="s" />\n')
        if self.isadmin or self.isfriend:
            self.stdout.write(
                '  ' + self.message["send"] +
                ' <input type="checkbox" value="error" name="error"' + 
                ' checked="checked" tabindex="2" accesskey="e" />\n')
        else:
            self.stdout.write(
                ' <input type="hidden" value="error" name="error"/>\n')
        self.stdout.write(
            '  ' + self.message["error"] + "<br />\n" +
            '  <textarea rows="20" cols="75" name="body"' +
            ' tabindex="8" accesskey="c">' + body + "</textarea>\n" +
            '  <input type="hidden" name="base_stamp"' +
            ' value="' + stamp + '" />\n' +
            '  <input type="hidden" name="base_id"' +
            ' value="' + id + '" />\n' +
            '</p></form>\n')
